package org.seasar.security.authorization;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.ListIterator;

import org.aopalliance.intercept.MethodInvocation;
import org.seasar.framework.aop.interceptors.AbstractInterceptor;
import org.seasar.framework.util.ArrayUtil;
import org.seasar.security.authentication.AuthenticationContext;
import org.seasar.security.authentication.AuthenticationContextResolver;

/**
 * @author t-wada
 */
public abstract class AbstractAuthorizationInterceptor extends AbstractInterceptor {
    private String[] roleNames_ = new String[0];
    private AuthenticationContextResolver contextResolver_;
    
    protected abstract Object ifInRole(MethodInvocation invocation, AuthenticationContext authContext, String roleName) throws Throwable;
    protected abstract Object ifNotInRole(MethodInvocation invocation, AuthenticationContext authContext) throws Throwable;
    
    public Object invoke(MethodInvocation invocation) throws Throwable {
        AuthenticationContext authContext = getResolver().resolve();
        for (int i = 0; i < this.roleNames_.length; i++) {
            String roleName = this.roleNames_[i];
            if(authContext.isUserInRole(roleName)) {
                return ifInRole(invocation, authContext, roleName);
            }
        }
        return ifNotInRole(invocation, authContext);
    }
    
    public void addRoleName(String roleName) {
        if(roleName == null) {
            return;
        }
        if(ArrayUtil.indexOf(this.roleNames_, roleName) == -1) {
            this.roleNames_ = (String[]) ArrayUtil.add(this.roleNames_, roleName);
        }
    }
    public void addRoleNames(List roleNames) {
        if(roleNames == null) {
            return;
        }
        List namesToAdd = new ArrayList();
        for (ListIterator iter = roleNames.listIterator(); iter.hasNext();) {
            String roleName = (String) iter.next();
            if(ArrayUtil.indexOf(this.roleNames_, roleName) == -1) {
                namesToAdd.add(roleName);
            }
        }
        if(namesToAdd.isEmpty()) {
            return;
        }
        List newRoleList = new ArrayList(Arrays.asList(this.roleNames_));
        newRoleList.addAll(namesToAdd);
        this.roleNames_ = (String[]) newRoleList.toArray(new String[newRoleList.size()]);
    }
    
    public void setAuthenticationContextResolver(AuthenticationContextResolver contextResolver) {
        this.contextResolver_ = contextResolver;
    }
    private AuthenticationContextResolver getResolver() {
        return this.contextResolver_;
    }
    
    
    public List getRoleNameList() {
        return new ArrayList(Arrays.asList(this.roleNames_));
    }
}
