package org.seasar.security.authorization;

import org.aopalliance.intercept.MethodInvocation;
import org.jmock.Mock;
import org.jmock.MockObjectTestCase;
import org.seasar.security.authentication.AuthenticationContext;
import org.seasar.security.authentication.AuthenticationContextResolver;

/**
 * @author t-wada
 */
public class AuthenticatedAccessOnlyInterceptorMockTest extends MockObjectTestCase {
    private Mock mockAuthContext;
    private Mock mockInvocation;
    private Mock mockResolver;
    
    protected void setUp() throws Exception {
    	mockAuthContext = new Mock(AuthenticationContext.class);
    	mockInvocation = new Mock(MethodInvocation.class);
        mockResolver = new Mock(AuthenticationContextResolver.class);
        mockResolver.expects(once()).method("resolve").will(returnValue((AuthenticationContext) mockAuthContext.proxy()));
    }
    
    private Object invokeTest() throws Throwable {
        mockInvocation.stubs().method("proceed");
    	AuthenticatedAccessOnlyInterceptor interceptor = new AuthenticatedAccessOnlyInterceptor();
    	interceptor.setAuthenticationContextResolver((AuthenticationContextResolver) mockResolver.proxy());
    	return interceptor.invoke((MethodInvocation) mockInvocation.proxy());
    }
    
    
    public void testAllowAccessIfAuthenticated() throws Throwable {
        // ## Arrange ##
        mockAuthContext.expects(once()).method("isAuthenticated").will(returnValue(true));
        
        // ## Act ##
        invokeTest();
        
        // ## Assert ##
        mockAuthContext.verify();
    }
    
    public void testThrowsSNotAuthenticatedExceptionIfNotAuthenticated() throws Throwable {
        // ## Arrange ##
        mockAuthContext.expects(once()).method("isAuthenticated").will(returnValue(false));
        Object targetComponent = "target component";
        mockInvocation.expects(once()).method("getThis").will(returnValue(targetComponent));
        try {
            // ## Act ##
            invokeTest();
            
            // ## Assert ##
            fail("SNotAuthenticatedException should be thrown");
        } catch (SNotAuthenticatedException expected) {
            String message = expected.getMessage();
            assertTrue("message should be replaced", message.indexOf("target component") != -1);
            assertSame(targetComponent, expected.getTargetComponent());
        }
        mockAuthContext.verify();
    }
}
